/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/ 
/*
  Copyright (C) 1998-1999 Johan Redestig
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
****************************************************************************/
#include "stdafx.h"

#include "PopupMaker.h"
#include "TaskMenu.h"
#include "TaskItem.h"

#include "HeaderItem.h"
#include "SeparatorItem.h"
#include "MenuItem.h"
#include "TaskFolder.h"

#define TASK_MENU_TIMER 4435

IgnoreList* tm_ignoreTag;

int TaskMenu::TaskMenuTimerInterval = 2000;
int TaskMenu::TaskMenuFilterLevel = 3; 
BOOL TaskMenu::sort = FALSE;
int TaskMenu::numIgnore = 0;


TaskMenu::TaskMenu(const char *pszTitle, PopupMaker *pPopupMaker)
		: PopupMenu(pPopupMaker->hInst)
{
	m_pPopupMaker = pPopupMaker;
	m_pszTitle = pszTitle ? strdup(pszTitle) : 0;
	ReadPopupTaskMenuIgnore();
}

TaskMenu::~TaskMenu()
{
	if (m_pszTitle)
		free(m_pszTitle);
}

void TaskMenu::OnShow(BOOL fShow)
{
	//Hide(HIDE_CHILDREN);
	//Hide(HIDE_PARENTS);
	if (fShow)
	{
		UpdateFolder();
		if (TaskMenuTimerInterval > 0)
			SetTimer(GetWindow(), TASK_MENU_TIMER, TaskMenuTimerInterval, NULL);
	}
	else if (TaskMenuTimerInterval > 0)
	{
			KillTimer(GetWindow(), TASK_MENU_TIMER);
	}
}

void TaskMenu::OnTimer(int nTimer)
{
	if (TaskMenuTimerInterval > 0)
	{
		if (nTimer == TASK_MENU_TIMER)
			UpdateFolder();
	}
}

void TaskMenu::UpdateFolder()
{
	LPSTR pszItem = "!PopupTasks";
	LPSTR pszTitle = "All Windows";


	DeleteMenuItems();
#ifndef _MINI
	m_pPopupMaker->AddHeaderItem(this, m_pszTitle, FALSE);
#else
	MenuItem* pMenuItem = m_pPopupMaker->MakeBangItem("!DateTime","   %y%m%d  %#I:%M %p");
	pMenuItem->SetActivePainter(m_pPopupMaker->m_pSelEntry);
	pMenuItem->SetPainter(m_pPopupMaker->m_pEntry);
	pMenuItem->SetHeight(m_pPopupMaker->m_nSubmenuHeight);
	AddMenuItem(pMenuItem);
	m_pPopupMaker->AddSeparator(this);
#endif /* _MINI */
	EnumWindows(EnumWindowsProc, (LPARAM) this);
	// still gotta add a subfolder (or functionality) with:
		// - arrange windows
		// - minimize all
		// - restore all
		// -- another subfolder showing All Tasks
#ifndef _MINI
	m_pPopupMaker->AddBottomItem(this);
#else
/*	m_pPopupMaker->AddSeparator(this);
	m_pPopupMaker->AddTaskFolder(this);  */
#endif /* _MINI */

	if (sort)
		Sort();
	Invalidate();
	Validate();
}

void TaskMenu::ReadPopupTaskMenuIgnore()
{
	FILE * f = LCOpen(NULL);
	if (!f)
		return;
	unsigned int i;
	char * token[2];
	char buffer[4096];
	numIgnore = 0;

	for (i=0;i<2;++i)
		token[i] = new char[1024];
	while (LCReadNextConfig (f, "*PopupTasksMenuIgnore", buffer, sizeof(buffer)))
	{
		int count;
		count = LCTokenize (buffer, token, 2, NULL);
		if (!StripWS(token[1])) continue;
		if (count >= 2)
		{
			if (!tm_ignoreTag)
				tm_ignoreTag = (IgnoreList *)malloc(sizeof(IgnoreList));
			else
				tm_ignoreTag = (IgnoreList *)realloc(tm_ignoreTag, (numIgnore+1)*sizeof(IgnoreList));
			tm_ignoreTag[numIgnore].match = new char[strlen(token[1])+1];
			strcpy(tm_ignoreTag[numIgnore].match, token[1]);
			++numIgnore;
		}
	}
	LCClose(f);
	for (i = 0; i < 2; ++i)
		delete [] token[i];
	return;
}

BOOL TaskMenu::EnumWindowsProc(HWND hWnd, LPARAM lParam)
{

	if (IsAppWindowWithFilter(hWnd, TaskMenuFilterLevel))
	{
		TaskMenu *menu = (TaskMenu *) lParam;

		if ( (numIgnore > 0) && ( TaskMenuFilterLevel > 1 ) )
		{
			char title[1024];
			GetWindowText(hWnd, title, sizeof(title) );
			
			for (int i=0; i</*menu->m_pPopupMaker->*/numIgnore; ++i)
			{
				if (strstr(title, tm_ignoreTag[i].match))
					return TRUE;
			}
		}
		TaskItem *item = new TaskItem(hWnd);

		item->SetActivePainter(menu->m_pPopupMaker->m_pSelEntry);
		item->SetPainter(menu->m_pPopupMaker->m_pEntry);
		item->SetHeight(menu->m_pPopupMaker->m_nSubmenuHeight);

		if (menu->m_pPopupMaker->m_bTaskIcons)
			item->SetIcon(GetIconFromWindowWithTimeout(hWnd, FALSE,menu->m_pPopupMaker->m_nGetIconTimeout));

		menu->AddMenuItem(item);
	}

	return TRUE;
}
