/*
This is a part of the LiteStep Shell Source code.

Copyright (C) 1997-2001 The LiteStep Development Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/****************************************************************************
****************************************************************************/
#include "contextmenu.h"

/////////////////////////////////////////////////////////////////////////////
// Context menu functions


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  FUNCTION:       DoExplorerMenu
//
//  DESCRIPTION:    Given a path name to a file or folder object, displays
//                  the shell's context menu for that object and executes
//                  the menu command (if any) selected by the user.
//
//  INPUT:          pszPath = Pointer to an ANSI or Unicode string
//                            specifying the path to the object.
//
//
//  RETURNS:        TRUE if successful, FALSE if not.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

BOOL PopupContextMenu(LPCSTR pszPath)
{
	LPMALLOC pMalloc;
	LPSHELLFOLDER psfFolder, psfNextFolder;
	LPITEMIDLIST pidlMain, pidlItem, pidlNextItem, *ppidl;
	LPCONTEXTMENU pContextMenu;
	CMINVOKECOMMANDINFO ici;
	ULONG ulCount, ulAttr;
	char tchPath[MAX_PATH];
	WCHAR wchPath[MAX_PATH];
	UINT nCount, nCmd;
	BOOL bResult;
	HMENU hMenu;

	// Get current cursor position & window
	POINT point;
	HWND hwnd;

	GetCursorPos(&point);
	hwnd = WindowFromPoint(point);

	/*
	  char szGotPath [MAX_PATH];
	 if (strstr (pszPath, ".lnk") != NULL)
	 {
	   OutputDebugString("lnk file found");
	   HRESULT hres;
	   IShellLink *psl;
	   WIN32_FIND_DATA wfd;
	   
	   // Get a pointer to the IShellLink interface.
	   hres = CoCreateInstance (CLSID_ShellLink, NULL,
	     CLSCTX_INPROC_SERVER,
	     IID_IShellLink, (void **)&psl);
	   if (SUCCEEDED (hres))
	   {
	     OutputDebugString("CoCreateInstance succeeded");
	     IPersistFile *ppf;
	     
	     // Get a pointer to the IPersistFile interface.
	     hres = psl->QueryInterface (IID_IPersistFile, (void
	       **)&ppf);
	     
	     if (SUCCEEDED (hres))
	     {
	       OutputDebugString("QueryInterface succeeded");
	       WORD wsz [MAX_PATH]; // buffer for Unicode string
	       
	       // Ensure that the string consists of Unicode characters.
	       MultiByteToWideChar (CP_ACP, 0, pszPath, -1, wsz,
	         MAX_PATH);
	       
	       // Load the shortcut.
	       hres = ppf->Load (wsz, STGM_READ);
	       
	       if (SUCCEEDED (hres))
	       {
	         OutputDebugString("Load succeeded");
	         // Resolve the shortcut.
	         hres = psl->Resolve (hwnd, SLR_ANY_MATCH);
	         if (SUCCEEDED (hres))
	         {
	           OutputDebugString("Resolve succeeded");
	           strcpy (szGotPath, pszPath);
	           // Get the path to the shortcut target.
	           hres = psl->GetPath (szGotPath, MAX_PATH,
	             (WIN32_FIND_DATA *)&wfd, SLGP_SHORTPATH);
	           OutputDebugString(szGotPath);
	         }
	       }
	       // Release the pointer to IPersistFile.
	       ppf->Release ();
	     }
	     // Release the pointer to IShellLink.
	     psl->Release ();
	   }
	 }
	 else
	 {
	   strcpy(szGotPath, pszPath);
	 }
	 */ 
	//
	// Make sure the file name is fully qualified and in Unicode format.
	//
	GetFullPathName (pszPath, sizeof (tchPath) / sizeof (char), tchPath, NULL);
	//GetFullPathName (szGotPath, sizeof (tchPath) / sizeof (char), tchPath, NULL);
	if (IsTextUnicode (tchPath, lstrlen (tchPath), NULL))
		lstrcpy ((char *) wchPath, tchPath);
	else
		MultiByteToWideChar (CP_ACP, 0, tchPath, -1, wchPath,
		                     sizeof (wchPath) / sizeof (WCHAR));

	//
	// Get pointers to the shell's IMalloc interface and the desktop's
	// IShellFolder interface.
	//
	bResult = FALSE;

	if (!SUCCEEDED (SHGetMalloc (&pMalloc)))
		return bResult;

	if (!SUCCEEDED (SHGetDesktopFolder (&psfFolder)))
	{
		pMalloc->Release();
		return bResult;
	}
	//
	// Convert the path name into a pointer to an item ID list (pidl).
	//
	if (SUCCEEDED (psfFolder->ParseDisplayName (hwnd, NULL, wchPath, &ulCount,
	               &pidlMain, &ulAttr)) && (pidlMain != NULL))
	{

		if (nCount = GetItemCount (pidlMain))
		{ // nCount must be > 0
			//
			// Initialize psfFolder with a pointer to the IShellFolder
			// interface of the folder that contains the item whose context
			// menu we're after, and initialize pidlItem with a pointer to
			// the item's item ID. If nCount > 1, this requires us to walk
			// the list of item IDs stored in pidlMain and bind to each
			// subfolder referenced in the list.
			//
			pidlItem = pidlMain;

			while (--nCount)
			{
				//
				// Create a 1-item item ID list for the next item in pidlMain.
				//
				pidlNextItem = DuplicateItem (pMalloc, pidlItem);
				if (pidlNextItem == NULL)
				{
					pMalloc->Free (pidlMain);
					psfFolder->Release();
					pMalloc->Release();
					return bResult;
				}

				//
				// Bind to the folder specified in the new item ID list.
				//
				if (!SUCCEEDED (psfFolder->BindToObject (pidlNextItem, NULL, IID_IShellFolder, (void**)&psfNextFolder)))
				{
					pMalloc->Free(pidlNextItem);
					pMalloc->Free (pidlMain);
					psfFolder->Release();
					pMalloc->Release();
					return bResult;
				}

				//
				// Release the IShellFolder pointer to the parent folder
				// and set psfFolder equal to the IShellFolder pointer for
				// the current folder.
				//
				psfFolder->Release();
				psfFolder = psfNextFolder;

				//
				// Release the storage for the 1-item item ID list we created
				// just a moment ago and initialize pidlItem so that it points
				// to the next item in pidlMain.
				//
				pMalloc->Free(pidlNextItem);
				pidlItem = GetNextItem (pidlItem);
			}

			//
			// Get a pointer to the item's IContextMenu interface and call
			// IContextMenu::QueryContextMenu to initialize a context menu.
			//
			ppidl = &pidlItem;
			if (SUCCEEDED (psfFolder->GetUIObjectOf (hwnd, 1, (const ITEMIDLIST**)ppidl, (const GUID)IID_IContextMenu, NULL, (void**)&pContextMenu)))
			{

				hMenu = CreatePopupMenu ();
				if (SUCCEEDED (pContextMenu->QueryContextMenu (
				                   hMenu, 0, 1, 0x7FFF, CMF_NORMAL)))
				{

					//
					// Display the context menu.
					//
					nCmd = TrackPopupMenu (hMenu, TPM_LEFTALIGN |
					                       TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD,
					                       point.x, point.y, 0, hwnd, NULL);

					//
					// If a command was selected from the menu, execute it.
					//
					if (nCmd)
					{
						ici.cbSize = sizeof (CMINVOKECOMMANDINFO);
						ici.fMask = SEE_MASK_FLAG_NO_UI;
						ici.hwnd = hwnd;
						ici.lpVerb = MAKEINTRESOURCE (nCmd - 1);
						ici.lpParameters = NULL;
						ici.lpDirectory = NULL;
						ici.nShow = SW_SHOWNORMAL;
						ici.dwHotKey = 0;
						ici.hIcon = NULL;

						if (SUCCEEDED (
						            pContextMenu->InvokeCommand (&ici)))
							bResult = TRUE;
					}
				}
				DestroyMenu (hMenu);
				pContextMenu->Release();
			}
		}
		pMalloc->Free (pidlMain);
	}

	//
	// Clean up and return.
	//
	psfFolder->Release();
	pMalloc->Release();

	return bResult;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  FUNCTION:       GetItemCount
//
//  DESCRIPTION:    Computes the number of item IDs in an item ID list.
//
//  INPUT:          pidl = Pointer to an item ID list.
//
//  RETURNS:        Number of item IDs in the list.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

UINT GetItemCount (LPITEMIDLIST pidl)
{
	USHORT nLen;
	UINT nCount;
	nCount = 0;
	while ((nLen = pidl->mkid.cb) != 0)
	{
		pidl = GetNextItem (pidl);
		++nCount;
	}
	return nCount;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  FUNCTION:       GetNextItem
//
//  DESCRIPTION:    Finds the next item in an item ID list.
//
//  INPUT:          pidl = Pointer to an item ID list.
//
//  RETURNS:        Pointer to the next item.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

LPITEMIDLIST GetNextItem (LPITEMIDLIST pidl)
{
	USHORT nLen;

	if ((nLen = pidl->mkid.cb) == 0)
		return NULL;

	return (LPITEMIDLIST) (((LPBYTE) pidl) + nLen);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  FUNCTION:       DuplicateItem
//
//  DESCRIPTION:    Makes a copy of the next item in an item ID list.
//
//  INPUT:          pMalloc = Pointer to an IMalloc interface.
//                  pidl    = Pointer to an item ID list.
//
//  RETURNS:        Pointer to an ITEMIDLIST containing the copied item ID.
//
//  NOTES:          It is the caller's responsibility to free the memory
//                  allocated by this function when the item ID is no longer
//                  needed. Example:
//
//                    pidlItem = DuplicateItem (pMalloc, pidl);
//                      .
//                      .
//                      .
//                    pMalloc->Free(pidlItem);
//
//                  Failure to free the ITEMIDLIST will result in memory
//                  leaks.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

LPITEMIDLIST DuplicateItem (LPMALLOC pMalloc, LPITEMIDLIST pidl)
{
	USHORT nLen;
	LPITEMIDLIST pidlNew;

	nLen = pidl->mkid.cb;
	if (nLen == 0)
		return NULL;

	pidlNew = (LPITEMIDLIST) pMalloc->Alloc (nLen + sizeof (USHORT));
	if (pidlNew == NULL)
		return NULL;

	CopyMemory (pidlNew, pidl, nLen);
	*((USHORT*) (((LPBYTE) pidlNew) + nLen)) = 0;

	return pidlNew;
}
